# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class PacketMirroring(pulumi.CustomResource):
    collector_ilb: pulumi.Output[dict]
    """
    The Forwarding Rule resource (of type load_balancing_scheme=INTERNAL)
    that will be used as collector for mirrored traffic. The
    specified forwarding rule must have is_mirroring_collector
    set to true.  Structure is documented below.

      * `url` (`str`) - The URL of the instances where this rule should be active.
    """
    description: pulumi.Output[str]
    """
    A human-readable description of the rule.
    """
    filter: pulumi.Output[dict]
    """
    A filter for mirrored traffic.  If unset, all traffic is mirrored.  Structure is documented below.

      * `cidrRanges` (`list`) - IP CIDR ranges that apply as a filter on the source (ingress) or
        destination (egress) IP in the IP header. Only IPv4 is supported.
      * `ipProtocols` (`list`) - Protocols that apply as a filter on mirrored traffic.
    """
    mirrored_resources: pulumi.Output[dict]
    """
    A means of specifying which resources to mirror.  Structure is documented below.

      * `instances` (`list`) - All the listed instances will be mirrored.  Specify at most 50.  Structure is documented below.
        * `url` (`str`) - The URL of the instances where this rule should be active.

      * `subnetworks` (`list`) - All instances in one of these subnetworks will be mirrored.  Structure is documented below.
        * `url` (`str`) - The URL of the instances where this rule should be active.

      * `tags` (`list`) - All instances with these tags will be mirrored.
    """
    name: pulumi.Output[str]
    """
    The name of the packet mirroring rule
    """
    network: pulumi.Output[dict]
    """
    Specifies the mirrored VPC network. Only packets in this network
    will be mirrored. All mirrored VMs should have a NIC in the given
    network. All mirrored subnetworks should belong to the given network.  Structure is documented below.

      * `url` (`str`) - The URL of the instances where this rule should be active.
    """
    priority: pulumi.Output[float]
    """
    Since only one rule can be active at a time, priority is
    used to break ties in the case of two rules that apply to
    the same instances.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    The Region in which the created address should reside.
    If it is not provided, the provider region is used.
    """
    def __init__(__self__, resource_name, opts=None, collector_ilb=None, description=None, filter=None, mirrored_resources=None, name=None, network=None, priority=None, project=None, region=None, __props__=None, __name__=None, __opts__=None):
        """
        Packet Mirroring mirrors traffic to and from particular VM instances.
        You can use the collected traffic to help you detect security threats
        and monitor application performance.

        To get more information about PacketMirroring, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/packetMirroring)
        * How-to Guides
            * [Using Packet Mirroring](https://cloud.google.com/vpc/docs/using-packet-mirroring#creating)

        ## Example Usage

        ### Compute Packet Mirroring Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default_network = gcp.compute.Network("defaultNetwork")
        mirror = gcp.compute.Instance("mirror",
            machine_type="n1-standard-1",
            boot_disk={
                "initialize_params": {
                    "image": "debian-cloud/debian-9",
                },
            },
            network_interface=[{
                "network": default_network.id,
                "access_config": [{}],
            }])
        default_subnetwork = gcp.compute.Subnetwork("defaultSubnetwork",
            network=default_network.id,
            ip_cidr_range="10.2.0.0/16")
        default_health_check = gcp.compute.HealthCheck("defaultHealthCheck",
            check_interval_sec=1,
            timeout_sec=1,
            tcp_health_check={
                "port": "80",
            })
        default_region_backend_service = gcp.compute.RegionBackendService("defaultRegionBackendService", health_checks=[default_health_check.id])
        default_forwarding_rule = gcp.compute.ForwardingRule("defaultForwardingRule",
            is_mirroring_collector=True,
            ip_protocol="TCP",
            load_balancing_scheme="INTERNAL",
            backend_service=default_region_backend_service.id,
            all_ports=True,
            network=default_network.id,
            subnetwork=default_subnetwork.id,
            network_tier="PREMIUM")
        foobar = gcp.compute.PacketMirroring("foobar",
            description="bar",
            network={
                "url": default_network.id,
            },
            collector_ilb={
                "url": default_forwarding_rule.id,
            },
            mirrored_resources={
                "tags": ["foo"],
                "instances": [{
                    "url": mirror.id,
                }],
            },
            filter={
                "ipProtocols": ["tcp"],
                "cidrRanges": ["0.0.0.0/0"],
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] collector_ilb: The Forwarding Rule resource (of type load_balancing_scheme=INTERNAL)
               that will be used as collector for mirrored traffic. The
               specified forwarding rule must have is_mirroring_collector
               set to true.  Structure is documented below.
        :param pulumi.Input[str] description: A human-readable description of the rule.
        :param pulumi.Input[dict] filter: A filter for mirrored traffic.  If unset, all traffic is mirrored.  Structure is documented below.
        :param pulumi.Input[dict] mirrored_resources: A means of specifying which resources to mirror.  Structure is documented below.
        :param pulumi.Input[str] name: The name of the packet mirroring rule
        :param pulumi.Input[dict] network: Specifies the mirrored VPC network. Only packets in this network
               will be mirrored. All mirrored VMs should have a NIC in the given
               network. All mirrored subnetworks should belong to the given network.  Structure is documented below.
        :param pulumi.Input[float] priority: Since only one rule can be active at a time, priority is
               used to break ties in the case of two rules that apply to
               the same instances.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created address should reside.
               If it is not provided, the provider region is used.

        The **collector_ilb** object supports the following:

          * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.

        The **filter** object supports the following:

          * `cidrRanges` (`pulumi.Input[list]`) - IP CIDR ranges that apply as a filter on the source (ingress) or
            destination (egress) IP in the IP header. Only IPv4 is supported.
          * `ipProtocols` (`pulumi.Input[list]`) - Protocols that apply as a filter on mirrored traffic.

        The **mirrored_resources** object supports the following:

          * `instances` (`pulumi.Input[list]`) - All the listed instances will be mirrored.  Specify at most 50.  Structure is documented below.
            * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.

          * `subnetworks` (`pulumi.Input[list]`) - All instances in one of these subnetworks will be mirrored.  Structure is documented below.
            * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.

          * `tags` (`pulumi.Input[list]`) - All instances with these tags will be mirrored.

        The **network** object supports the following:

          * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if collector_ilb is None:
                raise TypeError("Missing required property 'collector_ilb'")
            __props__['collector_ilb'] = collector_ilb
            __props__['description'] = description
            __props__['filter'] = filter
            if mirrored_resources is None:
                raise TypeError("Missing required property 'mirrored_resources'")
            __props__['mirrored_resources'] = mirrored_resources
            __props__['name'] = name
            if network is None:
                raise TypeError("Missing required property 'network'")
            __props__['network'] = network
            __props__['priority'] = priority
            __props__['project'] = project
            __props__['region'] = region
        super(PacketMirroring, __self__).__init__(
            'gcp:compute/packetMirroring:PacketMirroring',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, collector_ilb=None, description=None, filter=None, mirrored_resources=None, name=None, network=None, priority=None, project=None, region=None):
        """
        Get an existing PacketMirroring resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] collector_ilb: The Forwarding Rule resource (of type load_balancing_scheme=INTERNAL)
               that will be used as collector for mirrored traffic. The
               specified forwarding rule must have is_mirroring_collector
               set to true.  Structure is documented below.
        :param pulumi.Input[str] description: A human-readable description of the rule.
        :param pulumi.Input[dict] filter: A filter for mirrored traffic.  If unset, all traffic is mirrored.  Structure is documented below.
        :param pulumi.Input[dict] mirrored_resources: A means of specifying which resources to mirror.  Structure is documented below.
        :param pulumi.Input[str] name: The name of the packet mirroring rule
        :param pulumi.Input[dict] network: Specifies the mirrored VPC network. Only packets in this network
               will be mirrored. All mirrored VMs should have a NIC in the given
               network. All mirrored subnetworks should belong to the given network.  Structure is documented below.
        :param pulumi.Input[float] priority: Since only one rule can be active at a time, priority is
               used to break ties in the case of two rules that apply to
               the same instances.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created address should reside.
               If it is not provided, the provider region is used.

        The **collector_ilb** object supports the following:

          * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.

        The **filter** object supports the following:

          * `cidrRanges` (`pulumi.Input[list]`) - IP CIDR ranges that apply as a filter on the source (ingress) or
            destination (egress) IP in the IP header. Only IPv4 is supported.
          * `ipProtocols` (`pulumi.Input[list]`) - Protocols that apply as a filter on mirrored traffic.

        The **mirrored_resources** object supports the following:

          * `instances` (`pulumi.Input[list]`) - All the listed instances will be mirrored.  Specify at most 50.  Structure is documented below.
            * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.

          * `subnetworks` (`pulumi.Input[list]`) - All instances in one of these subnetworks will be mirrored.  Structure is documented below.
            * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.

          * `tags` (`pulumi.Input[list]`) - All instances with these tags will be mirrored.

        The **network** object supports the following:

          * `url` (`pulumi.Input[str]`) - The URL of the instances where this rule should be active.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["collector_ilb"] = collector_ilb
        __props__["description"] = description
        __props__["filter"] = filter
        __props__["mirrored_resources"] = mirrored_resources
        __props__["name"] = name
        __props__["network"] = network
        __props__["priority"] = priority
        __props__["project"] = project
        __props__["region"] = region
        return PacketMirroring(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class InstanceGroupNamedPort(pulumi.CustomResource):
    group: pulumi.Output[str]
    """
    The name of the instance group.
    """
    name: pulumi.Output[str]
    """
    The name for this named port. The name must be 1-63 characters
    long, and comply with RFC1035.
    """
    port: pulumi.Output[float]
    """
    The port number, which can be a value between 1 and 65535.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    zone: pulumi.Output[str]
    """
    The zone of the instance group.
    """
    def __init__(__self__, resource_name, opts=None, group=None, name=None, port=None, project=None, zone=None, __props__=None, __name__=None, __opts__=None):
        """
        Mange the named ports setting for a managed instance group without
        managing the group as whole. This resource is primarily intended for use
        with GKE-generated groups that shouldn't otherwise be managed by other
        tools.


        To get more information about InstanceGroupNamedPort, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/instanceGroup)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/compute/docs/instance-groups/)

        ## Example Usage

        ### Instance Group Named Port Gke

        ```python
        import pulumi
        import pulumi_gcp as gcp

        container_network = gcp.compute.Network("containerNetwork", auto_create_subnetworks=False)
        container_subnetwork = gcp.compute.Subnetwork("containerSubnetwork",
            region="us-central1",
            network=container_network.name,
            ip_cidr_range="10.0.36.0/24")
        my_cluster = gcp.container.Cluster("myCluster",
            location="us-central1-a",
            initial_node_count=1,
            network=container_network.name,
            subnetwork=container_subnetwork.name,
            ip_allocation_policy={
                "clusterIpv4CidrBlock": "/19",
                "servicesIpv4CidrBlock": "/22",
            })
        my_port = gcp.compute.InstanceGroupNamedPort("myPort",
            group=my_cluster.instance_group_urls[0],
            zone="us-central1-a",
            port=8080)
        my_ports = gcp.compute.InstanceGroupNamedPort("myPorts",
            group=my_cluster.instance_group_urls[0],
            zone="us-central1-a",
            port=4443)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] group: The name of the instance group.
        :param pulumi.Input[str] name: The name for this named port. The name must be 1-63 characters
               long, and comply with RFC1035.
        :param pulumi.Input[float] port: The port number, which can be a value between 1 and 65535.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] zone: The zone of the instance group.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if group is None:
                raise TypeError("Missing required property 'group'")
            __props__['group'] = group
            __props__['name'] = name
            if port is None:
                raise TypeError("Missing required property 'port'")
            __props__['port'] = port
            __props__['project'] = project
            __props__['zone'] = zone
        super(InstanceGroupNamedPort, __self__).__init__(
            'gcp:compute/instanceGroupNamedPort:InstanceGroupNamedPort',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, group=None, name=None, port=None, project=None, zone=None):
        """
        Get an existing InstanceGroupNamedPort resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] group: The name of the instance group.
        :param pulumi.Input[str] name: The name for this named port. The name must be 1-63 characters
               long, and comply with RFC1035.
        :param pulumi.Input[float] port: The port number, which can be a value between 1 and 65535.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] zone: The zone of the instance group.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["group"] = group
        __props__["name"] = name
        __props__["port"] = port
        __props__["project"] = project
        __props__["zone"] = zone
        return InstanceGroupNamedPort(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class RegionHealthCheck(pulumi.CustomResource):
    check_interval_sec: pulumi.Output[float]
    """
    How often (in seconds) to send a health check. The default value is 5
    seconds.
    """
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    An optional description of this resource. Provide this property when
    you create the resource.
    """
    healthy_threshold: pulumi.Output[float]
    """
    A so-far unhealthy instance will be marked healthy after this many
    consecutive successes. The default value is 2.
    """
    http2_health_check: pulumi.Output[dict]
    """
    A nested object resource  Structure is documented below.

      * `host` (`str`) - The value of the host header in the HTTP2 health check request.
        If left empty (default value), the public IP on behalf of which this health
        check is performed will be used.
      * `port` (`float`) - The TCP port number for the HTTP2 health check request.
        The default value is 443.
      * `port_name` (`str`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
        port_name are defined, port takes precedence.
      * `portSpecification` (`str`) - Specifies how port is selected for health checking, can be one of the
        following values:
        * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
        * `USE_NAMED_PORT`: The `portName` is used for health checking.
        * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the
        port or named port specified in the Backend Service is used for health
        checking.
        If not specified, HTTP2 health check follows behavior specified in `port` and
        `portName` fields.
      * `proxy_header` (`str`) - Specifies the type of proxy header to append before sending data to the
        backend.
      * `request_path` (`str`) - The request path of the HTTP2 health check request.
        The default value is /.
      * `response` (`str`) - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
    """
    http_health_check: pulumi.Output[dict]
    """
    A nested object resource  Structure is documented below.

      * `host` (`str`) - The value of the host header in the HTTP2 health check request.
        If left empty (default value), the public IP on behalf of which this health
        check is performed will be used.
      * `port` (`float`) - The TCP port number for the HTTP2 health check request.
        The default value is 443.
      * `port_name` (`str`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
        port_name are defined, port takes precedence.
      * `portSpecification` (`str`) - Specifies how port is selected for health checking, can be one of the
        following values:
        * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
        * `USE_NAMED_PORT`: The `portName` is used for health checking.
        * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the
        port or named port specified in the Backend Service is used for health
        checking.
        If not specified, HTTP2 health check follows behavior specified in `port` and
        `portName` fields.
      * `proxy_header` (`str`) - Specifies the type of proxy header to append before sending data to the
        backend.
      * `request_path` (`str`) - The request path of the HTTP2 health check request.
        The default value is /.
      * `response` (`str`) - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
    """
    https_health_check: pulumi.Output[dict]
    """
    A nested object resource  Structure is documented below.

      * `host` (`str`) - The value of the host header in the HTTP2 health check request.
        If left empty (default value), the public IP on behalf of which this health
        check is performed will be used.
      * `port` (`float`) - The TCP port number for the HTTP2 health check request.
        The default value is 443.
      * `port_name` (`str`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
        port_name are defined, port takes precedence.
      * `portSpecification` (`str`) - Specifies how port is selected for health checking, can be one of the
        following values:
        * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
        * `USE_NAMED_PORT`: The `portName` is used for health checking.
        * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the
        port or named port specified in the Backend Service is used for health
        checking.
        If not specified, HTTP2 health check follows behavior specified in `port` and
        `portName` fields.
      * `proxy_header` (`str`) - Specifies the type of proxy header to append before sending data to the
        backend.
      * `request_path` (`str`) - The request path of the HTTP2 health check request.
        The default value is /.
      * `response` (`str`) - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
    """
    log_config: pulumi.Output[dict]
    """
    Configure logging on this health check.  Structure is documented below.

      * `enable` (`bool`) - Indicates whether or not to export logs. This is false by default,
        which means no health check logging will be done.
    """
    name: pulumi.Output[str]
    """
    Name of the resource. Provided by the client when the resource is
    created. The name must be 1-63 characters long, and comply with
    RFC1035.  Specifically, the name must be 1-63 characters long and
    match the regular expression `a-z?` which means
    the first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the
    last character, which cannot be a dash.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    The Region in which the created health check should reside.
    If it is not provided, the provider region is used.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    ssl_health_check: pulumi.Output[dict]
    """
    A nested object resource  Structure is documented below.

      * `port` (`float`) - The TCP port number for the HTTP2 health check request.
        The default value is 443.
      * `port_name` (`str`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
        port_name are defined, port takes precedence.
      * `portSpecification` (`str`) - Specifies how port is selected for health checking, can be one of the
        following values:
        * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
        * `USE_NAMED_PORT`: The `portName` is used for health checking.
        * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the
        port or named port specified in the Backend Service is used for health
        checking.
        If not specified, HTTP2 health check follows behavior specified in `port` and
        `portName` fields.
      * `proxy_header` (`str`) - Specifies the type of proxy header to append before sending data to the
        backend.
      * `request` (`str`) - The application data to send once the SSL connection has been
        established (default value is empty). If both request and response are
        empty, the connection establishment alone will indicate health. The request
        data can only be ASCII.
      * `response` (`str`) - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
    """
    tcp_health_check: pulumi.Output[dict]
    """
    A nested object resource  Structure is documented below.

      * `port` (`float`) - The TCP port number for the HTTP2 health check request.
        The default value is 443.
      * `port_name` (`str`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
        port_name are defined, port takes precedence.
      * `portSpecification` (`str`) - Specifies how port is selected for health checking, can be one of the
        following values:
        * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
        * `USE_NAMED_PORT`: The `portName` is used for health checking.
        * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the
        port or named port specified in the Backend Service is used for health
        checking.
        If not specified, HTTP2 health check follows behavior specified in `port` and
        `portName` fields.
      * `proxy_header` (`str`) - Specifies the type of proxy header to append before sending data to the
        backend.
      * `request` (`str`) - The application data to send once the SSL connection has been
        established (default value is empty). If both request and response are
        empty, the connection establishment alone will indicate health. The request
        data can only be ASCII.
      * `response` (`str`) - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
    """
    timeout_sec: pulumi.Output[float]
    """
    How long (in seconds) to wait before claiming failure.
    The default value is 5 seconds.  It is invalid for timeoutSec to have
    greater value than checkIntervalSec.
    """
    type: pulumi.Output[str]
    """
    The type of the health check. One of HTTP, HTTP2, HTTPS, TCP, or SSL.
    """
    unhealthy_threshold: pulumi.Output[float]
    """
    A so-far healthy instance will be marked unhealthy after this many
    consecutive failures. The default value is 2.
    """
    def __init__(__self__, resource_name, opts=None, check_interval_sec=None, description=None, healthy_threshold=None, http2_health_check=None, http_health_check=None, https_health_check=None, log_config=None, name=None, project=None, region=None, ssl_health_check=None, tcp_health_check=None, timeout_sec=None, unhealthy_threshold=None, __props__=None, __name__=None, __opts__=None):
        """
        Health Checks determine whether instances are responsive and able to do work.
        They are an important part of a comprehensive load balancing configuration,
        as they enable monitoring instances behind load balancers.

        Health Checks poll instances at a specified interval. Instances that
        do not respond successfully to some number of probes in a row are marked
        as unhealthy. No new connections are sent to unhealthy instances,
        though existing connections will continue. The health check will
        continue to poll unhealthy instances. If an instance later responds
        successfully to some number of consecutive probes, it is marked
        healthy again and can receive new connections.


        To get more information about RegionHealthCheck, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/regionHealthChecks)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/load-balancing/docs/health-checks)

        ## Example Usage

        ### Region Health Check Tcp

        ```python
        import pulumi
        import pulumi_gcp as gcp

        tcp_region_health_check = gcp.compute.RegionHealthCheck("tcp-region-health-check",
            check_interval_sec=1,
            tcp_health_check={
                "port": "80",
            },
            timeout_sec=1)
        ```

        ### Region Health Check Tcp Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        tcp_region_health_check = gcp.compute.RegionHealthCheck("tcp-region-health-check",
            check_interval_sec=1,
            description="Health check via tcp",
            healthy_threshold=4,
            tcp_health_check={
                "port_name": "health-check-port",
                "portSpecification": "USE_NAMED_PORT",
                "proxy_header": "NONE",
                "request": "ARE YOU HEALTHY?",
                "response": "I AM HEALTHY",
            },
            timeout_sec=1,
            unhealthy_threshold=5)
        ```

        ### Region Health Check Ssl

        ```python
        import pulumi
        import pulumi_gcp as gcp

        ssl_region_health_check = gcp.compute.RegionHealthCheck("ssl-region-health-check",
            check_interval_sec=1,
            ssl_health_check={
                "port": "443",
            },
            timeout_sec=1)
        ```

        ### Region Health Check Ssl Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        ssl_region_health_check = gcp.compute.RegionHealthCheck("ssl-region-health-check",
            check_interval_sec=1,
            description="Health check via ssl",
            healthy_threshold=4,
            ssl_health_check={
                "port_name": "health-check-port",
                "portSpecification": "USE_NAMED_PORT",
                "proxy_header": "NONE",
                "request": "ARE YOU HEALTHY?",
                "response": "I AM HEALTHY",
            },
            timeout_sec=1,
            unhealthy_threshold=5)
        ```

        ### Region Health Check Http

        ```python
        import pulumi
        import pulumi_gcp as gcp

        http_region_health_check = gcp.compute.RegionHealthCheck("http-region-health-check",
            check_interval_sec=1,
            http_health_check={
                "port": "80",
            },
            timeout_sec=1)
        ```

        ### Region Health Check Http Logs

        ```python
        import pulumi
        import pulumi_gcp as gcp

        http_region_health_check = gcp.compute.RegionHealthCheck("http-region-health-check",
            timeout_sec=1,
            check_interval_sec=1,
            http_health_check={
                "port": "80",
            },
            log_config={
                "enable": True,
            })
        ```

        ### Region Health Check Http Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        http_region_health_check = gcp.compute.RegionHealthCheck("http-region-health-check",
            check_interval_sec=1,
            description="Health check via http",
            healthy_threshold=4,
            http_health_check={
                "host": "1.2.3.4",
                "port_name": "health-check-port",
                "portSpecification": "USE_NAMED_PORT",
                "proxy_header": "NONE",
                "request_path": "/mypath",
                "response": "I AM HEALTHY",
            },
            timeout_sec=1,
            unhealthy_threshold=5)
        ```

        ### Region Health Check Https

        ```python
        import pulumi
        import pulumi_gcp as gcp

        https_region_health_check = gcp.compute.RegionHealthCheck("https-region-health-check",
            check_interval_sec=1,
            https_health_check={
                "port": "443",
            },
            timeout_sec=1)
        ```

        ### Region Health Check Https Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        https_region_health_check = gcp.compute.RegionHealthCheck("https-region-health-check",
            check_interval_sec=1,
            description="Health check via https",
            healthy_threshold=4,
            https_health_check={
                "host": "1.2.3.4",
                "port_name": "health-check-port",
                "portSpecification": "USE_NAMED_PORT",
                "proxy_header": "NONE",
                "request_path": "/mypath",
                "response": "I AM HEALTHY",
            },
            timeout_sec=1,
            unhealthy_threshold=5)
        ```

        ### Region Health Check Http2

        ```python
        import pulumi
        import pulumi_gcp as gcp

        http2_region_health_check = gcp.compute.RegionHealthCheck("http2-region-health-check",
            check_interval_sec=1,
            http2_health_check={
                "port": "443",
            },
            timeout_sec=1)
        ```

        ### Region Health Check Http2 Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        http2_region_health_check = gcp.compute.RegionHealthCheck("http2-region-health-check",
            check_interval_sec=1,
            description="Health check via http2",
            healthy_threshold=4,
            http2_health_check={
                "host": "1.2.3.4",
                "port_name": "health-check-port",
                "portSpecification": "USE_NAMED_PORT",
                "proxy_header": "NONE",
                "request_path": "/mypath",
                "response": "I AM HEALTHY",
            },
            timeout_sec=1,
            unhealthy_threshold=5)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] check_interval_sec: How often (in seconds) to send a health check. The default value is 5
               seconds.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when
               you create the resource.
        :param pulumi.Input[float] healthy_threshold: A so-far unhealthy instance will be marked healthy after this many
               consecutive successes. The default value is 2.
        :param pulumi.Input[dict] http2_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] http_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] https_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] log_config: Configure logging on this health check.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the
               last character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created health check should reside.
               If it is not provided, the provider region is used.
        :param pulumi.Input[dict] ssl_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] tcp_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[float] timeout_sec: How long (in seconds) to wait before claiming failure.
               The default value is 5 seconds.  It is invalid for timeoutSec to have
               greater value than checkIntervalSec.
        :param pulumi.Input[float] unhealthy_threshold: A so-far healthy instance will be marked unhealthy after this many
               consecutive failures. The default value is 2.

        The **http2_health_check** object supports the following:

          * `host` (`pulumi.Input[str]`) - The value of the host header in the HTTP2 health check request.
            If left empty (default value), the public IP on behalf of which this health
            check is performed will be used.
          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request_path` (`pulumi.Input[str]`) - The request path of the HTTP2 health check request.
            The default value is /.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **http_health_check** object supports the following:

          * `host` (`pulumi.Input[str]`) - The value of the host header in the HTTP2 health check request.
            If left empty (default value), the public IP on behalf of which this health
            check is performed will be used.
          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request_path` (`pulumi.Input[str]`) - The request path of the HTTP2 health check request.
            The default value is /.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **https_health_check** object supports the following:

          * `host` (`pulumi.Input[str]`) - The value of the host header in the HTTP2 health check request.
            If left empty (default value), the public IP on behalf of which this health
            check is performed will be used.
          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request_path` (`pulumi.Input[str]`) - The request path of the HTTP2 health check request.
            The default value is /.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **log_config** object supports the following:

          * `enable` (`pulumi.Input[bool]`) - Indicates whether or not to export logs. This is false by default,
            which means no health check logging will be done.

        The **ssl_health_check** object supports the following:

          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request` (`pulumi.Input[str]`) - The application data to send once the SSL connection has been
            established (default value is empty). If both request and response are
            empty, the connection establishment alone will indicate health. The request
            data can only be ASCII.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **tcp_health_check** object supports the following:

          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request` (`pulumi.Input[str]`) - The application data to send once the SSL connection has been
            established (default value is empty). If both request and response are
            empty, the connection establishment alone will indicate health. The request
            data can only be ASCII.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['check_interval_sec'] = check_interval_sec
            __props__['description'] = description
            __props__['healthy_threshold'] = healthy_threshold
            __props__['http2_health_check'] = http2_health_check
            __props__['http_health_check'] = http_health_check
            __props__['https_health_check'] = https_health_check
            __props__['log_config'] = log_config
            __props__['name'] = name
            __props__['project'] = project
            __props__['region'] = region
            __props__['ssl_health_check'] = ssl_health_check
            __props__['tcp_health_check'] = tcp_health_check
            __props__['timeout_sec'] = timeout_sec
            __props__['unhealthy_threshold'] = unhealthy_threshold
            __props__['creation_timestamp'] = None
            __props__['self_link'] = None
            __props__['type'] = None
        super(RegionHealthCheck, __self__).__init__(
            'gcp:compute/regionHealthCheck:RegionHealthCheck',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, check_interval_sec=None, creation_timestamp=None, description=None, healthy_threshold=None, http2_health_check=None, http_health_check=None, https_health_check=None, log_config=None, name=None, project=None, region=None, self_link=None, ssl_health_check=None, tcp_health_check=None, timeout_sec=None, type=None, unhealthy_threshold=None):
        """
        Get an existing RegionHealthCheck resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] check_interval_sec: How often (in seconds) to send a health check. The default value is 5
               seconds.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when
               you create the resource.
        :param pulumi.Input[float] healthy_threshold: A so-far unhealthy instance will be marked healthy after this many
               consecutive successes. The default value is 2.
        :param pulumi.Input[dict] http2_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] http_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] https_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] log_config: Configure logging on this health check.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the
               last character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created health check should reside.
               If it is not provided, the provider region is used.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        :param pulumi.Input[dict] ssl_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[dict] tcp_health_check: A nested object resource  Structure is documented below.
        :param pulumi.Input[float] timeout_sec: How long (in seconds) to wait before claiming failure.
               The default value is 5 seconds.  It is invalid for timeoutSec to have
               greater value than checkIntervalSec.
        :param pulumi.Input[str] type: The type of the health check. One of HTTP, HTTP2, HTTPS, TCP, or SSL.
        :param pulumi.Input[float] unhealthy_threshold: A so-far healthy instance will be marked unhealthy after this many
               consecutive failures. The default value is 2.

        The **http2_health_check** object supports the following:

          * `host` (`pulumi.Input[str]`) - The value of the host header in the HTTP2 health check request.
            If left empty (default value), the public IP on behalf of which this health
            check is performed will be used.
          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request_path` (`pulumi.Input[str]`) - The request path of the HTTP2 health check request.
            The default value is /.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **http_health_check** object supports the following:

          * `host` (`pulumi.Input[str]`) - The value of the host header in the HTTP2 health check request.
            If left empty (default value), the public IP on behalf of which this health
            check is performed will be used.
          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request_path` (`pulumi.Input[str]`) - The request path of the HTTP2 health check request.
            The default value is /.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **https_health_check** object supports the following:

          * `host` (`pulumi.Input[str]`) - The value of the host header in the HTTP2 health check request.
            If left empty (default value), the public IP on behalf of which this health
            check is performed will be used.
          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request_path` (`pulumi.Input[str]`) - The request path of the HTTP2 health check request.
            The default value is /.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **log_config** object supports the following:

          * `enable` (`pulumi.Input[bool]`) - Indicates whether or not to export logs. This is false by default,
            which means no health check logging will be done.

        The **ssl_health_check** object supports the following:

          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request` (`pulumi.Input[str]`) - The application data to send once the SSL connection has been
            established (default value is empty). If both request and response are
            empty, the connection establishment alone will indicate health. The request
            data can only be ASCII.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.

        The **tcp_health_check** object supports the following:

          * `port` (`pulumi.Input[float]`) - The TCP port number for the HTTP2 health check request.
            The default value is 443.
          * `port_name` (`pulumi.Input[str]`) - Port name as defined in InstanceGroup#NamedPort#name. If both port and
            port_name are defined, port takes precedence.
          * `portSpecification` (`pulumi.Input[str]`) - Specifies how port is selected for health checking, can be one of the
            following values:
            * `USE_FIXED_PORT`: The port number in `port` is used for health checking.
            * `USE_NAMED_PORT`: The `portName` is used for health checking.
            * `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
            network endpoint is used for health checking. For other backends, the
            port or named port specified in the Backend Service is used for health
            checking.
            If not specified, HTTP2 health check follows behavior specified in `port` and
            `portName` fields.
          * `proxy_header` (`pulumi.Input[str]`) - Specifies the type of proxy header to append before sending data to the
            backend.
          * `request` (`pulumi.Input[str]`) - The application data to send once the SSL connection has been
            established (default value is empty). If both request and response are
            empty, the connection establishment alone will indicate health. The request
            data can only be ASCII.
          * `response` (`pulumi.Input[str]`) - The bytes to match against the beginning of the response data. If left empty
            (the default value), any response will indicate health. The response data
            can only be ASCII.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["check_interval_sec"] = check_interval_sec
        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["healthy_threshold"] = healthy_threshold
        __props__["http2_health_check"] = http2_health_check
        __props__["http_health_check"] = http_health_check
        __props__["https_health_check"] = https_health_check
        __props__["log_config"] = log_config
        __props__["name"] = name
        __props__["project"] = project
        __props__["region"] = region
        __props__["self_link"] = self_link
        __props__["ssl_health_check"] = ssl_health_check
        __props__["tcp_health_check"] = tcp_health_check
        __props__["timeout_sec"] = timeout_sec
        __props__["type"] = type
        __props__["unhealthy_threshold"] = unhealthy_threshold
        return RegionHealthCheck(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class DiskResourcePolicyAttachment(pulumi.CustomResource):
    disk: pulumi.Output[str]
    """
    The name of the disk in which the resource policies are attached to.
    """
    name: pulumi.Output[str]
    """
    The resource policy to be attached to the disk for scheduling snapshot
    creation. Do not specify the self link.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    zone: pulumi.Output[str]
    """
    A reference to the zone where the disk resides.
    """
    def __init__(__self__, resource_name, opts=None, disk=None, name=None, project=None, zone=None, __props__=None, __name__=None, __opts__=None):
        """
        Adds existing resource policies to a disk. You can only add one policy
        which will be applied to this disk for scheduling snapshot creation.

        > **Note:** This resource does not support regional disks (`compute.RegionDisk`). For regional disks, please refer to the `compute.RegionDiskResourcePolicyAttachment` resource.


        ## Example Usage

        ### Disk Resource Policy Attachment Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        my_image = gcp.compute.get_image(family="debian-9",
            project="debian-cloud")
        ssd = gcp.compute.Disk("ssd",
            image=my_image.self_link,
            size=50,
            type="pd-ssd",
            zone="us-central1-a")
        attachment = gcp.compute.DiskResourcePolicyAttachment("attachment",
            disk=ssd.name,
            zone="us-central1-a")
        policy = gcp.compute.ResourcePolicy("policy",
            region="us-central1",
            snapshot_schedule_policy={
                "schedule": {
                    "daily_schedule": {
                        "daysInCycle": 1,
                        "startTime": "04:00",
                    },
                },
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] disk: The name of the disk in which the resource policies are attached to.
        :param pulumi.Input[str] name: The resource policy to be attached to the disk for scheduling snapshot
               creation. Do not specify the self link.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] zone: A reference to the zone where the disk resides.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if disk is None:
                raise TypeError("Missing required property 'disk'")
            __props__['disk'] = disk
            __props__['name'] = name
            __props__['project'] = project
            __props__['zone'] = zone
        super(DiskResourcePolicyAttachment, __self__).__init__(
            'gcp:compute/diskResourcePolicyAttachment:DiskResourcePolicyAttachment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, disk=None, name=None, project=None, zone=None):
        """
        Get an existing DiskResourcePolicyAttachment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] disk: The name of the disk in which the resource policies are attached to.
        :param pulumi.Input[str] name: The resource policy to be attached to the disk for scheduling snapshot
               creation. Do not specify the self link.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] zone: A reference to the zone where the disk resides.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["disk"] = disk
        __props__["name"] = name
        __props__["project"] = project
        __props__["zone"] = zone
        return DiskResourcePolicyAttachment(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class AutoscalingPolicy(pulumi.CustomResource):
    basic_algorithm: pulumi.Output[dict]
    """
    Basic algorithm for autoscaling.  Structure is documented below.

      * `cooldownPeriod` (`str`) - Duration between scaling events. A scaling period starts after the
        update operation from the previous event has completed.
        Bounds: [2m, 1d]. Default: 2m.
      * `yarnConfig` (`dict`) - YARN autoscaling configuration.  Structure is documented below.
        * `gracefulDecommissionTimeout` (`str`) - Timeout for YARN graceful decommissioning of Node Managers. Specifies the
          duration to wait for jobs to complete before forcefully removing workers
          (and potentially interrupting jobs). Only applicable to downscaling operations.
          Bounds: [0s, 1d].
        * `scaleDownFactor` (`float`) - Fraction of average pending memory in the last cooldown period for which to
          remove workers. A scale-down factor of 1 will result in scaling down so that there
          is no available memory remaining after the update (more aggressive scaling).
          A scale-down factor of 0 disables removing workers, which can be beneficial for
          autoscaling a single job.
          Bounds: [0.0, 1.0].
        * `scaleDownMinWorkerFraction` (`float`) - Minimum scale-down threshold as a fraction of total cluster size before scaling occurs.
          For example, in a 20-worker cluster, a threshold of 0.1 means the autoscaler must
          recommend at least a 2 worker scale-down for the cluster to scale. A threshold of 0
          means the autoscaler will scale down on any recommended change.
          Bounds: [0.0, 1.0]. Default: 0.0.
        * `scaleUpFactor` (`float`) - Fraction of average pending memory in the last cooldown period for which to
          add workers. A scale-up factor of 1.0 will result in scaling up so that there
          is no pending memory remaining after the update (more aggressive scaling).
          A scale-up factor closer to 0 will result in a smaller magnitude of scaling up
          (less aggressive scaling).
          Bounds: [0.0, 1.0].
        * `scaleUpMinWorkerFraction` (`float`) - Minimum scale-up threshold as a fraction of total cluster size before scaling
          occurs. For example, in a 20-worker cluster, a threshold of 0.1 means the autoscaler
          must recommend at least a 2-worker scale-up for the cluster to scale. A threshold of
          0 means the autoscaler will scale up on any recommended change.
          Bounds: [0.0, 1.0]. Default: 0.0.
    """
    location: pulumi.Output[str]
    """
    The  location where the autoscaling poicy should reside.
    The default value is `global`.
    """
    name: pulumi.Output[str]
    """
    The "resource name" of the autoscaling policy.
    """
    policy_id: pulumi.Output[str]
    """
    The policy id. The id must contain only letters (a-z, A-Z), numbers (0-9), underscores (_),
    and hyphens (-). Cannot begin or end with underscore or hyphen. Must consist of between
    3 and 50 characters.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    secondary_worker_config: pulumi.Output[dict]
    """
    Describes how the autoscaler will operate for secondary workers.  Structure is documented below.

      * `max_instances` (`float`) - Maximum number of instances for this group. Note that by default, clusters will not use
        secondary workers. Required for secondary workers if the minimum secondary instances is set.
        Bounds: [minInstances, ). Defaults to 0.
      * `minInstances` (`float`) - Minimum number of instances for this group. Bounds: [0, maxInstances]. Defaults to 0.
      * `weight` (`float`) - Weight for the instance group, which is used to determine the fraction of total workers
        in the cluster from this instance group. For example, if primary workers have weight 2,
        and secondary workers have weight 1, the cluster will have approximately 2 primary workers
        for each secondary worker.
        The cluster may not reach the specified balance if constrained by min/max bounds or other
        autoscaling settings. For example, if maxInstances for secondary workers is 0, then only
        primary workers will be added. The cluster can also be out of balance when created.
        If weight is not set on any instance group, the cluster will default to equal weight for
        all groups: the cluster will attempt to maintain an equal number of workers in each group
        within the configured size bounds for each group. If weight is set for one group only,
        the cluster will default to zero weight on the unset group. For example if weight is set
        only on primary workers, the cluster will use primary workers only and no secondary workers.
    """
    worker_config: pulumi.Output[dict]
    """
    Describes how the autoscaler will operate for primary workers.  Structure is documented below.

      * `max_instances` (`float`) - Maximum number of instances for this group. Note that by default, clusters will not use
        secondary workers. Required for secondary workers if the minimum secondary instances is set.
        Bounds: [minInstances, ). Defaults to 0.
      * `minInstances` (`float`) - Minimum number of instances for this group. Bounds: [0, maxInstances]. Defaults to 0.
      * `weight` (`float`) - Weight for the instance group, which is used to determine the fraction of total workers
        in the cluster from this instance group. For example, if primary workers have weight 2,
        and secondary workers have weight 1, the cluster will have approximately 2 primary workers
        for each secondary worker.
        The cluster may not reach the specified balance if constrained by min/max bounds or other
        autoscaling settings. For example, if maxInstances for secondary workers is 0, then only
        primary workers will be added. The cluster can also be out of balance when created.
        If weight is not set on any instance group, the cluster will default to equal weight for
        all groups: the cluster will attempt to maintain an equal number of workers in each group
        within the configured size bounds for each group. If weight is set for one group only,
        the cluster will default to zero weight on the unset group. For example if weight is set
        only on primary workers, the cluster will use primary workers only and no secondary workers.
    """
    def __init__(__self__, resource_name, opts=None, basic_algorithm=None, location=None, policy_id=None, project=None, secondary_worker_config=None, worker_config=None, __props__=None, __name__=None, __opts__=None):
        """
        Describes an autoscaling policy for Dataproc cluster autoscaler.



        ## Example Usage

        ### Dataproc Autoscaling Policy

        ```python
        import pulumi
        import pulumi_gcp as gcp

        asp = gcp.dataproc.AutoscalingPolicy("asp",
            policy_id="dataproc-policy",
            location="us-central1",
            worker_config={
                "max_instances": 3,
            },
            basic_algorithm={
                "yarn_config": {
                    "gracefulDecommissionTimeout": "30s",
                    "scaleUpFactor": 0.5,
                    "scaleDownFactor": 0.5,
                },
            })
        basic = gcp.dataproc.Cluster("basic",
            region="us-central1",
            cluster_config={
                "autoscaling_config": {
                    "policyUri": asp.name,
                },
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] basic_algorithm: Basic algorithm for autoscaling.  Structure is documented below.
        :param pulumi.Input[str] location: The  location where the autoscaling poicy should reside.
               The default value is `global`.
        :param pulumi.Input[str] policy_id: The policy id. The id must contain only letters (a-z, A-Z), numbers (0-9), underscores (_),
               and hyphens (-). Cannot begin or end with underscore or hyphen. Must consist of between
               3 and 50 characters.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] secondary_worker_config: Describes how the autoscaler will operate for secondary workers.  Structure is documented below.
        :param pulumi.Input[dict] worker_config: Describes how the autoscaler will operate for primary workers.  Structure is documented below.

        The **basic_algorithm** object supports the following:

          * `cooldownPeriod` (`pulumi.Input[str]`) - Duration between scaling events. A scaling period starts after the
            update operation from the previous event has completed.
            Bounds: [2m, 1d]. Default: 2m.
          * `yarnConfig` (`pulumi.Input[dict]`) - YARN autoscaling configuration.  Structure is documented below.
            * `gracefulDecommissionTimeout` (`pulumi.Input[str]`) - Timeout for YARN graceful decommissioning of Node Managers. Specifies the
              duration to wait for jobs to complete before forcefully removing workers
              (and potentially interrupting jobs). Only applicable to downscaling operations.
              Bounds: [0s, 1d].
            * `scaleDownFactor` (`pulumi.Input[float]`) - Fraction of average pending memory in the last cooldown period for which to
              remove workers. A scale-down factor of 1 will result in scaling down so that there
              is no available memory remaining after the update (more aggressive scaling).
              A scale-down factor of 0 disables removing workers, which can be beneficial for
              autoscaling a single job.
              Bounds: [0.0, 1.0].
            * `scaleDownMinWorkerFraction` (`pulumi.Input[float]`) - Minimum scale-down threshold as a fraction of total cluster size before scaling occurs.
              For example, in a 20-worker cluster, a threshold of 0.1 means the autoscaler must
              recommend at least a 2 worker scale-down for the cluster to scale. A threshold of 0
              means the autoscaler will scale down on any recommended change.
              Bounds: [0.0, 1.0]. Default: 0.0.
            * `scaleUpFactor` (`pulumi.Input[float]`) - Fraction of average pending memory in the last cooldown period for which to
              add workers. A scale-up factor of 1.0 will result in scaling up so that there
              is no pending memory remaining after the update (more aggressive scaling).
              A scale-up factor closer to 0 will result in a smaller magnitude of scaling up
              (less aggressive scaling).
              Bounds: [0.0, 1.0].
            * `scaleUpMinWorkerFraction` (`pulumi.Input[float]`) - Minimum scale-up threshold as a fraction of total cluster size before scaling
              occurs. For example, in a 20-worker cluster, a threshold of 0.1 means the autoscaler
              must recommend at least a 2-worker scale-up for the cluster to scale. A threshold of
              0 means the autoscaler will scale up on any recommended change.
              Bounds: [0.0, 1.0]. Default: 0.0.

        The **secondary_worker_config** object supports the following:

          * `max_instances` (`pulumi.Input[float]`) - Maximum number of instances for this group. Note that by default, clusters will not use
            secondary workers. Required for secondary workers if the minimum secondary instances is set.
            Bounds: [minInstances, ). Defaults to 0.
          * `minInstances` (`pulumi.Input[float]`) - Minimum number of instances for this group. Bounds: [0, maxInstances]. Defaults to 0.
          * `weight` (`pulumi.Input[float]`) - Weight for the instance group, which is used to determine the fraction of total workers
            in the cluster from this instance group. For example, if primary workers have weight 2,
            and secondary workers have weight 1, the cluster will have approximately 2 primary workers
            for each secondary worker.
            The cluster may not reach the specified balance if constrained by min/max bounds or other
            autoscaling settings. For example, if maxInstances for secondary workers is 0, then only
            primary workers will be added. The cluster can also be out of balance when created.
            If weight is not set on any instance group, the cluster will default to equal weight for
            all groups: the cluster will attempt to maintain an equal number of workers in each group
            within the configured size bounds for each group. If weight is set for one group only,
            the cluster will default to zero weight on the unset group. For example if weight is set
            only on primary workers, the cluster will use primary workers only and no secondary workers.

        The **worker_config** object supports the following:

          * `max_instances` (`pulumi.Input[float]`) - Maximum number of instances for this group. Note that by default, clusters will not use
            secondary workers. Required for secondary workers if the minimum secondary instances is set.
            Bounds: [minInstances, ). Defaults to 0.
          * `minInstances` (`pulumi.Input[float]`) - Minimum number of instances for this group. Bounds: [0, maxInstances]. Defaults to 0.
          * `weight` (`pulumi.Input[float]`) - Weight for the instance group, which is used to determine the fraction of total workers
            in the cluster from this instance group. For example, if primary workers have weight 2,
            and secondary workers have weight 1, the cluster will have approximately 2 primary workers
            for each secondary worker.
            The cluster may not reach the specified balance if constrained by min/max bounds or other
            autoscaling settings. For example, if maxInstances for secondary workers is 0, then only
            primary workers will be added. The cluster can also be out of balance when created.
            If weight is not set on any instance group, the cluster will default to equal weight for
            all groups: the cluster will attempt to maintain an equal number of workers in each group
            within the configured size bounds for each group. If weight is set for one group only,
            the cluster will default to zero weight on the unset group. For example if weight is set
            only on primary workers, the cluster will use primary workers only and no secondary workers.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['basic_algorithm'] = basic_algorithm
            __props__['location'] = location
            if policy_id is None:
                raise TypeError("Missing required property 'policy_id'")
            __props__['policy_id'] = policy_id
            __props__['project'] = project
            __props__['secondary_worker_config'] = secondary_worker_config
            __props__['worker_config'] = worker_config
            __props__['name'] = None
        super(AutoscalingPolicy, __self__).__init__(
            'gcp:dataproc/autoscalingPolicy:AutoscalingPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, basic_algorithm=None, location=None, name=None, policy_id=None, project=None, secondary_worker_config=None, worker_config=None):
        """
        Get an existing AutoscalingPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] basic_algorithm: Basic algorithm for autoscaling.  Structure is documented below.
        :param pulumi.Input[str] location: The  location where the autoscaling poicy should reside.
               The default value is `global`.
        :param pulumi.Input[str] name: The "resource name" of the autoscaling policy.
        :param pulumi.Input[str] policy_id: The policy id. The id must contain only letters (a-z, A-Z), numbers (0-9), underscores (_),
               and hyphens (-). Cannot begin or end with underscore or hyphen. Must consist of between
               3 and 50 characters.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] secondary_worker_config: Describes how the autoscaler will operate for secondary workers.  Structure is documented below.
        :param pulumi.Input[dict] worker_config: Describes how the autoscaler will operate for primary workers.  Structure is documented below.

        The **basic_algorithm** object supports the following:

          * `cooldownPeriod` (`pulumi.Input[str]`) - Duration between scaling events. A scaling period starts after the
            update operation from the previous event has completed.
            Bounds: [2m, 1d]. Default: 2m.
          * `yarnConfig` (`pulumi.Input[dict]`) - YARN autoscaling configuration.  Structure is documented below.
            * `gracefulDecommissionTimeout` (`pulumi.Input[str]`) - Timeout for YARN graceful decommissioning of Node Managers. Specifies the
              duration to wait for jobs to complete before forcefully removing workers
              (and potentially interrupting jobs). Only applicable to downscaling operations.
              Bounds: [0s, 1d].
            * `scaleDownFactor` (`pulumi.Input[float]`) - Fraction of average pending memory in the last cooldown period for which to
              remove workers. A scale-down factor of 1 will result in scaling down so that there
              is no available memory remaining after the update (more aggressive scaling).
              A scale-down factor of 0 disables removing workers, which can be beneficial for
              autoscaling a single job.
              Bounds: [0.0, 1.0].
            * `scaleDownMinWorkerFraction` (`pulumi.Input[float]`) - Minimum scale-down threshold as a fraction of total cluster size before scaling occurs.
              For example, in a 20-worker cluster, a threshold of 0.1 means the autoscaler must
              recommend at least a 2 worker scale-down for the cluster to scale. A threshold of 0
              means the autoscaler will scale down on any recommended change.
              Bounds: [0.0, 1.0]. Default: 0.0.
            * `scaleUpFactor` (`pulumi.Input[float]`) - Fraction of average pending memory in the last cooldown period for which to
              add workers. A scale-up factor of 1.0 will result in scaling up so that there
              is no pending memory remaining after the update (more aggressive scaling).
              A scale-up factor closer to 0 will result in a smaller magnitude of scaling up
              (less aggressive scaling).
              Bounds: [0.0, 1.0].
            * `scaleUpMinWorkerFraction` (`pulumi.Input[float]`) - Minimum scale-up threshold as a fraction of total cluster size before scaling
              occurs. For example, in a 20-worker cluster, a threshold of 0.1 means the autoscaler
              must recommend at least a 2-worker scale-up for the cluster to scale. A threshold of
              0 means the autoscaler will scale up on any recommended change.
              Bounds: [0.0, 1.0]. Default: 0.0.

        The **secondary_worker_config** object supports the following:

          * `max_instances` (`pulumi.Input[float]`) - Maximum number of instances for this group. Note that by default, clusters will not use
            secondary workers. Required for secondary workers if the minimum secondary instances is set.
            Bounds: [minInstances, ). Defaults to 0.
          * `minInstances` (`pulumi.Input[float]`) - Minimum number of instances for this group. Bounds: [0, maxInstances]. Defaults to 0.
          * `weight` (`pulumi.Input[float]`) - Weight for the instance group, which is used to determine the fraction of total workers
            in the cluster from this instance group. For example, if primary workers have weight 2,
            and secondary workers have weight 1, the cluster will have approximately 2 primary workers
            for each secondary worker.
            The cluster may not reach the specified balance if constrained by min/max bounds or other
            autoscaling settings. For example, if maxInstances for secondary workers is 0, then only
            primary workers will be added. The cluster can also be out of balance when created.
            If weight is not set on any instance group, the cluster will default to equal weight for
            all groups: the cluster will attempt to maintain an equal number of workers in each group
            within the configured size bounds for each group. If weight is set for one group only,
            the cluster will default to zero weight on the unset group. For example if weight is set
            only on primary workers, the cluster will use primary workers only and no secondary workers.

        The **worker_config** object supports the following:

          * `max_instances` (`pulumi.Input[float]`) - Maximum number of instances for this group. Note that by default, clusters will not use
            secondary workers. Required for secondary workers if the minimum secondary instances is set.
            Bounds: [minInstances, ). Defaults to 0.
          * `minInstances` (`pulumi.Input[float]`) - Minimum number of instances for this group. Bounds: [0, maxInstances]. Defaults to 0.
          * `weight` (`pulumi.Input[float]`) - Weight for the instance group, which is used to determine the fraction of total workers
            in the cluster from this instance group. For example, if primary workers have weight 2,
            and secondary workers have weight 1, the cluster will have approximately 2 primary workers
            for each secondary worker.
            The cluster may not reach the specified balance if constrained by min/max bounds or other
            autoscaling settings. For example, if maxInstances for secondary workers is 0, then only
            primary workers will be added. The cluster can also be out of balance when created.
            If weight is not set on any instance group, the cluster will default to equal weight for
            all groups: the cluster will attempt to maintain an equal number of workers in each group
            within the configured size bounds for each group. If weight is set for one group only,
            the cluster will default to zero weight on the unset group. For example if weight is set
            only on primary workers, the cluster will use primary workers only and no secondary workers.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["basic_algorithm"] = basic_algorithm
        __props__["location"] = location
        __props__["name"] = name
        __props__["policy_id"] = policy_id
        __props__["project"] = project
        __props__["secondary_worker_config"] = secondary_worker_config
        __props__["worker_config"] = worker_config
        return AutoscalingPolicy(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Job(pulumi.CustomResource):
    copy: pulumi.Output[dict]
    """
    Copies a table.  Structure is documented below.

      * `createDisposition` (`str`) - Specifies whether the job is allowed to create new tables. The following values are supported:
        CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
        CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
        Creation, truncation and append actions occur as one atomic update upon job completion
      * `destinationEncryptionConfiguration` (`dict`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
        * `kms_key_name` (`str`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
          The BigQuery Service Account associated with your project requires access to this encryption key.

      * `destinationTable` (`dict`) - The destination table.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `project_id` (`str`) - The ID of the project containing this model.
        * `table_id` (`str`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
          or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

      * `sourceTables` (`list`) - Source tables to copy.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `project_id` (`str`) - The ID of the project containing this model.
        * `table_id` (`str`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
          or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

      * `writeDisposition` (`str`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
        WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
        WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
        WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
        Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
        Creation, truncation and append actions occur as one atomic update upon job completion.
    """
    extract: pulumi.Output[dict]
    """
    Configures an extract job.  Structure is documented below.

      * `compression` (`str`) - The compression type to use for exported files. Possible values include GZIP, DEFLATE, SNAPPY, and NONE.
        The default value is NONE. DEFLATE and SNAPPY are only supported for Avro.
      * `destinationFormat` (`str`) - The exported file format. Possible values include CSV, NEWLINE_DELIMITED_JSON and AVRO for tables and SAVED_MODEL for models.
        The default value for tables is CSV. Tables with nested or repeated fields cannot be exported as CSV.
        The default value for models is SAVED_MODEL.
      * `destinationUris` (`list`) - A list of fully-qualified Google Cloud Storage URIs where the extracted table should be written.
      * `fieldDelimiter` (`str`) - When extracting data in CSV format, this defines the delimiter to use between fields in the exported data.
        Default is ','
      * `printHeader` (`bool`) - Whether to print out a header row in the results. Default is true.
      * `sourceModel` (`dict`) - A reference to the model being exported.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `modelId` (`str`) - The ID of the model.
        * `project_id` (`str`) - The ID of the project containing this model.

      * `sourceTable` (`dict`) - A reference to the table being exported.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `project_id` (`str`) - The ID of the project containing this model.
        * `table_id` (`str`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
          or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

      * `useAvroLogicalTypes` (`bool`) - Whether to use logical types when extracting to AVRO format.
    """
    job_id: pulumi.Output[str]
    """
    The ID of the job. The ID must contain only letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). The maximum length is 1,024 characters.
    """
    job_timeout_ms: pulumi.Output[str]
    """
    Job timeout in milliseconds. If this time limit is exceeded, BigQuery may attempt to terminate the job.
    """
    job_type: pulumi.Output[str]
    """
    The type of the job.
    """
    labels: pulumi.Output[dict]
    """
    The labels associated with this job. You can use these to organize and group your jobs.
    """
    load: pulumi.Output[dict]
    """
    Configures a load job.  Structure is documented below.

      * `allowJaggedRows` (`bool`) - Accept rows that are missing trailing optional columns. The missing values are treated as nulls.
        If false, records with missing trailing columns are treated as bad records, and if there are too many bad records,
        an invalid error is returned in the job result. The default value is false. Only applicable to CSV, ignored for other formats.
      * `allowQuotedNewlines` (`bool`) - Indicates if BigQuery should allow quoted data sections that contain newline characters in a CSV file.
        The default value is false.
      * `autodetect` (`bool`) - Indicates if we should automatically infer the options and schema for CSV and JSON sources.
      * `createDisposition` (`str`) - Specifies whether the job is allowed to create new tables. The following values are supported:
        CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
        CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
        Creation, truncation and append actions occur as one atomic update upon job completion
      * `destinationEncryptionConfiguration` (`dict`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
        * `kms_key_name` (`str`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
          The BigQuery Service Account associated with your project requires access to this encryption key.

      * `destinationTable` (`dict`) - The destination table.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `project_id` (`str`) - The ID of the project containing this model.
        * `table_id` (`str`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
          or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

      * `encoding` (`str`) - The character encoding of the data. The supported values are UTF-8 or ISO-8859-1.
        The default value is UTF-8. BigQuery decodes the data after the raw, binary data
        has been split using the values of the quote and fieldDelimiter properties.
      * `fieldDelimiter` (`str`) - When extracting data in CSV format, this defines the delimiter to use between fields in the exported data.
        Default is ','
      * `ignoreUnknownValues` (`bool`) - Indicates if BigQuery should allow extra values that are not represented in the table schema.
        If true, the extra values are ignored. If false, records with extra columns are treated as bad records,
        and if there are too many bad records, an invalid error is returned in the job result.
        The default value is false. The sourceFormat property determines what BigQuery treats as an extra value:
        CSV: Trailing columns
        JSON: Named values that don't match any column names
      * `maxBadRecords` (`float`) - The maximum number of bad records that BigQuery can ignore when running the job. If the number of bad records exceeds this value,
        an invalid error is returned in the job result. The default value is 0, which requires that all records are valid.
      * `nullMarker` (`str`) - Specifies a string that represents a null value in a CSV file. The default value is the empty string. If you set this
        property to a custom value, BigQuery throws an error if an
        empty string is present for all data types except for STRING and BYTE. For STRING and BYTE columns, BigQuery interprets the empty string as
        an empty value.
      * `projectionFields` (`list`) - If sourceFormat is set to "DATASTORE_BACKUP", indicates which entity properties to load into BigQuery from a Cloud Datastore backup.
        Property names are case sensitive and must be top-level properties. If no properties are specified, BigQuery loads all properties.
        If any named property isn't found in the Cloud Datastore backup, an invalid error is returned in the job result.
      * `quote` (`str`) - The value that is used to quote data sections in a CSV file. BigQuery converts the string to ISO-8859-1 encoding,
        and then uses the first byte of the encoded string to split the data in its raw, binary state.
        The default value is a double-quote ('"'). If your data does not contain quoted sections, set the property value to an empty string.
        If your data contains quoted newline characters, you must also set the allowQuotedNewlines property to true.
      * `schemaUpdateOptions` (`list`) - Allows the schema of the destination table to be updated as a side effect of the load job if a schema is autodetected or
        supplied in the job configuration. Schema update options are supported in two cases: when writeDisposition is WRITE_APPEND;
        when writeDisposition is WRITE_TRUNCATE and the destination table is a partition of a table, specified by partition decorators.
        For normal tables, WRITE_TRUNCATE will always overwrite the schema. One or more of the following values are specified:
        ALLOW_FIELD_ADDITION: allow adding a nullable field to the schema.
        ALLOW_FIELD_RELAXATION: allow relaxing a required field in the original schema to nullable.
      * `skipLeadingRows` (`float`) - The number of rows at the top of a CSV file that BigQuery will skip when loading the data.
        The default value is 0. This property is useful if you have header rows in the file that should be skipped.
        When autodetect is on, the behavior is the following:
        skipLeadingRows unspecified - Autodetect tries to detect headers in the first row. If they are not detected,
        the row is read as data. Otherwise data is read starting from the second row.
        skipLeadingRows is 0 - Instructs autodetect that there are no headers and data should be read starting from the first row.
        skipLeadingRows = N > 0 - Autodetect skips N-1 rows and tries to detect headers in row N. If headers are not detected,
        row N is just skipped. Otherwise row N is used to extract column names for the detected schema.
      * `sourceFormat` (`str`) - The format of the data files. For CSV files, specify "CSV". For datastore backups, specify "DATASTORE_BACKUP".
        For newline-delimited JSON, specify "NEWLINE_DELIMITED_JSON". For Avro, specify "AVRO". For parquet, specify "PARQUET".
        For orc, specify "ORC". The default value is CSV.
      * `sourceUris` (`list`) - The fully-qualified URIs that point to your data in Google Cloud.
        For Google Cloud Storage URIs: Each URI can contain one '*' wildcard character
        and it must come after the 'bucket' name. Size limits related to load jobs apply
        to external data sources. For Google Cloud Bigtable URIs: Exactly one URI can be
        specified and it has be a fully specified and valid HTTPS URL for a Google Cloud Bigtable table.
        For Google Cloud Datastore backups: Exactly one URI can be specified. Also, the '*' wildcard character is not allowed.
      * `time_partitioning` (`dict`) - Time-based partitioning specification for the destination table.  Structure is documented below.
        * `expirationMs` (`str`) - Number of milliseconds for which to keep the storage for a partition. A wrapper is used here because 0 is an invalid value.
        * `field` (`str`) - If not set, the table is partitioned by pseudo column '_PARTITIONTIME'; if set, the table is partitioned by this field.
          The field must be a top-level TIMESTAMP or DATE field. Its mode must be NULLABLE or REQUIRED.
          A wrapper is used here because an empty string is an invalid value.
        * `type` (`str`) - The only type supported is DAY, which will generate one partition per day. Providing an empty string used to cause an error,
          but in OnePlatform the field will be treated as unset.

      * `writeDisposition` (`str`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
        WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
        WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
        WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
        Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
        Creation, truncation and append actions occur as one atomic update upon job completion.
    """
    location: pulumi.Output[str]
    """
    The geographic location of the job. The default value is US.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    query: pulumi.Output[dict]
    """
    Configures a query job.  Structure is documented below.

      * `allowLargeResults` (`bool`) - If true and query uses legacy SQL dialect, allows the query to produce arbitrarily large result tables at a slight cost in performance.
        Requires destinationTable to be set. For standard SQL queries, this flag is ignored and large results are always allowed.
        However, you must still set destinationTable when result size exceeds the allowed maximum response size.
      * `createDisposition` (`str`) - Specifies whether the job is allowed to create new tables. The following values are supported:
        CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
        CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
        Creation, truncation and append actions occur as one atomic update upon job completion
      * `defaultDataset` (`dict`) - Specifies the default dataset to use for unqualified table names in the query. Note that this does not alter behavior of unqualified dataset names.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `project_id` (`str`) - The ID of the project containing this model.

      * `destinationEncryptionConfiguration` (`dict`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
        * `kms_key_name` (`str`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
          The BigQuery Service Account associated with your project requires access to this encryption key.

      * `destinationTable` (`dict`) - The destination table.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this model.
        * `project_id` (`str`) - The ID of the project containing this model.
        * `table_id` (`str`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
          or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

      * `flattenResults` (`bool`) - If true and query uses legacy SQL dialect, flattens all nested and repeated fields in the query results.
        allowLargeResults must be true if this is set to false. For standard SQL queries, this flag is ignored and results are never flattened.
      * `maximumBillingTier` (`float`) - Limits the billing tier for this job. Queries that have resource usage beyond this tier will fail (without incurring a charge).
        If unspecified, this will be set to your project default.
      * `maximumBytesBilled` (`str`) - Limits the bytes billed for this job. Queries that will have bytes billed beyond this limit will fail (without incurring a charge).
        If unspecified, this will be set to your project default.
      * `parameterMode` (`str`) - Standard SQL only. Set to POSITIONAL to use positional (?) query parameters or to NAMED to use named (@myparam) query parameters in this query.
      * `priority` (`str`) - Specifies a priority for the query.
      * `query` (`str`) - Configures a query job.  Structure is documented below.
      * `schemaUpdateOptions` (`list`) - Allows the schema of the destination table to be updated as a side effect of the load job if a schema is autodetected or
        supplied in the job configuration. Schema update options are supported in two cases: when writeDisposition is WRITE_APPEND;
        when writeDisposition is WRITE_TRUNCATE and the destination table is a partition of a table, specified by partition decorators.
        For normal tables, WRITE_TRUNCATE will always overwrite the schema. One or more of the following values are specified:
        ALLOW_FIELD_ADDITION: allow adding a nullable field to the schema.
        ALLOW_FIELD_RELAXATION: allow relaxing a required field in the original schema to nullable.
      * `scriptOptions` (`dict`) - Options controlling the execution of scripts.  Structure is documented below.
        * `keyResultStatement` (`str`) - Determines which statement in the script represents the "key result",
          used to populate the schema and query results of the script job.
        * `statementByteBudget` (`str`) - Limit on the number of bytes billed per statement. Exceeding this budget results in an error.
        * `statementTimeoutMs` (`str`) - Timeout period for each statement in a script.

      * `useLegacySql` (`bool`) - Specifies whether to use BigQuery's legacy SQL dialect for this query. The default value is true.
        If set to false, the query will use BigQuery's standard SQL.
      * `useQueryCache` (`bool`) - Whether to look for the result in the query cache. The query cache is a best-effort cache that will be flushed whenever
        tables in the query are modified. Moreover, the query cache is only available when a query does not have a destination table specified.
        The default value is true.
      * `userDefinedFunctionResources` (`list`) - Describes user-defined function resources used in the query.  Structure is documented below.
        * `inlineCode` (`str`) - An inline resource that contains code for a user-defined function (UDF).
          Providing a inline code resource is equivalent to providing a URI for a file containing the same code.
        * `resource_uri` (`str`) - A code resource to load from a Google Cloud Storage URI (gs://bucket/path).

      * `writeDisposition` (`str`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
        WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
        WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
        WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
        Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
        Creation, truncation and append actions occur as one atomic update upon job completion.
    """
    user_email: pulumi.Output[str]
    """
    Email address of the user who ran the job.
    """
    def __init__(__self__, resource_name, opts=None, copy=None, extract=None, job_id=None, job_timeout_ms=None, labels=None, load=None, location=None, project=None, query=None, __props__=None, __name__=None, __opts__=None):
        """
        Jobs are actions that BigQuery runs on your behalf to load data, export data, query data, or copy data.
        Once a BigQuery job is created, it cannot be changed or deleted.



        ## Example Usage

        ### Bigquery Job Query

        ```python
        import pulumi
        import pulumi_gcp as gcp

        bar = gcp.bigquery.Dataset("bar",
            dataset_id="job_query_dataset",
            friendly_name="test",
            description="This is a test description",
            location="US")
        foo = gcp.bigquery.Table("foo",
            dataset_id=bar.dataset_id,
            table_id="job_query_table")
        job = gcp.bigquery.Job("job",
            job_id="job_query",
            labels={
                "example-label": "example-value",
            },
            query={
                "query": "SELECT state FROM [lookerdata:cdc.project_tycho_reports]",
                "destination_table": {
                    "project_id": foo.project,
                    "dataset_id": foo.dataset_id,
                    "table_id": foo.table_id,
                },
                "allowLargeResults": True,
                "flattenResults": True,
                "script_options": {
                    "keyResultStatement": "LAST",
                },
            })
        ```

        ### Bigquery Job Query Table Reference

        ```python
        import pulumi
        import pulumi_gcp as gcp

        bar = gcp.bigquery.Dataset("bar",
            dataset_id="job_query_dataset",
            friendly_name="test",
            description="This is a test description",
            location="US")
        foo = gcp.bigquery.Table("foo",
            dataset_id=bar.dataset_id,
            table_id="job_query_table")
        job = gcp.bigquery.Job("job",
            job_id="job_query",
            labels={
                "example-label": "example-value",
            },
            query={
                "query": "SELECT state FROM [lookerdata:cdc.project_tycho_reports]",
                "destination_table": {
                    "table_id": foo.id,
                },
                "default_dataset": {
                    "dataset_id": bar.id,
                },
                "allowLargeResults": True,
                "flattenResults": True,
                "script_options": {
                    "keyResultStatement": "LAST",
                },
            })
        ```

        ### Bigquery Job Load

        ```python
        import pulumi
        import pulumi_gcp as gcp

        bar = gcp.bigquery.Dataset("bar",
            dataset_id="job_load_dataset",
            friendly_name="test",
            description="This is a test description",
            location="US")
        foo = gcp.bigquery.Table("foo",
            dataset_id=bar.dataset_id,
            table_id="job_load_table")
        job = gcp.bigquery.Job("job",
            job_id="job_load",
            labels={
                "my_job": "load",
            },
            load={
                "sourceUris": ["gs://cloud-samples-data/bigquery/us-states/us-states-by-date.csv"],
                "destination_table": {
                    "project_id": foo.project,
                    "dataset_id": foo.dataset_id,
                    "table_id": foo.table_id,
                },
                "skipLeadingRows": 1,
                "schemaUpdateOptions": [
                    "ALLOW_FIELD_RELAXATION",
                    "ALLOW_FIELD_ADDITION",
                ],
                "writeDisposition": "WRITE_APPEND",
                "autodetect": True,
            })
        ```

        ### Bigquery Job Extract

        ```python
        import pulumi
        import pulumi_gcp as gcp

        source_one_dataset = gcp.bigquery.Dataset("source-oneDataset",
            dataset_id="job_extract_dataset",
            friendly_name="test",
            description="This is a test description",
            location="US")
        source_one_table = gcp.bigquery.Table("source-oneTable",
            dataset_id=source_one_dataset.dataset_id,
            table_id="job_extract_table",
            schema=\"\"\"[
          {
            "name": "name",
            "type": "STRING",
            "mode": "NULLABLE"
          },
          {
            "name": "post_abbr",
            "type": "STRING",
            "mode": "NULLABLE"
          },
          {
            "name": "date",
            "type": "DATE",
            "mode": "NULLABLE"
          }
        ]
        \"\"\")
        dest = gcp.storage.Bucket("dest", force_destroy=True)
        job = gcp.bigquery.Job("job",
            job_id="job_extract",
            extract={
                "destinationUris": [dest.url.apply(lambda url: f"{url}/extract")],
                "source_table": {
                    "project_id": source_one_table.project,
                    "dataset_id": source_one_table.dataset_id,
                    "table_id": source_one_table.table_id,
                },
                "destinationFormat": "NEWLINE_DELIMITED_JSON",
                "compression": "GZIP",
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] copy: Copies a table.  Structure is documented below.
        :param pulumi.Input[dict] extract: Configures an extract job.  Structure is documented below.
        :param pulumi.Input[str] job_id: The ID of the job. The ID must contain only letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). The maximum length is 1,024 characters.
        :param pulumi.Input[str] job_timeout_ms: Job timeout in milliseconds. If this time limit is exceeded, BigQuery may attempt to terminate the job.
        :param pulumi.Input[dict] labels: The labels associated with this job. You can use these to organize and group your jobs.
        :param pulumi.Input[dict] load: Configures a load job.  Structure is documented below.
        :param pulumi.Input[str] location: The geographic location of the job. The default value is US.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] query: Configures a query job.  Structure is documented below.

        The **copy** object supports the following:

          * `createDisposition` (`pulumi.Input[str]`) - Specifies whether the job is allowed to create new tables. The following values are supported:
            CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
            CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
            Creation, truncation and append actions occur as one atomic update upon job completion
          * `destinationEncryptionConfiguration` (`pulumi.Input[dict]`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
            * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
              The BigQuery Service Account associated with your project requires access to this encryption key.

          * `destinationTable` (`pulumi.Input[dict]`) - The destination table.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `sourceTables` (`pulumi.Input[list]`) - Source tables to copy.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `writeDisposition` (`pulumi.Input[str]`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
            WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
            WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
            WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
            Creation, truncation and append actions occur as one atomic update upon job completion.

        The **extract** object supports the following:

          * `compression` (`pulumi.Input[str]`) - The compression type to use for exported files. Possible values include GZIP, DEFLATE, SNAPPY, and NONE.
            The default value is NONE. DEFLATE and SNAPPY are only supported for Avro.
          * `destinationFormat` (`pulumi.Input[str]`) - The exported file format. Possible values include CSV, NEWLINE_DELIMITED_JSON and AVRO for tables and SAVED_MODEL for models.
            The default value for tables is CSV. Tables with nested or repeated fields cannot be exported as CSV.
            The default value for models is SAVED_MODEL.
          * `destinationUris` (`pulumi.Input[list]`) - A list of fully-qualified Google Cloud Storage URIs where the extracted table should be written.
          * `fieldDelimiter` (`pulumi.Input[str]`) - When extracting data in CSV format, this defines the delimiter to use between fields in the exported data.
            Default is ','
          * `printHeader` (`pulumi.Input[bool]`) - Whether to print out a header row in the results. Default is true.
          * `sourceModel` (`pulumi.Input[dict]`) - A reference to the model being exported.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `modelId` (`pulumi.Input[str]`) - The ID of the model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.

          * `sourceTable` (`pulumi.Input[dict]`) - A reference to the table being exported.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `useAvroLogicalTypes` (`pulumi.Input[bool]`) - Whether to use logical types when extracting to AVRO format.

        The **load** object supports the following:

          * `allowJaggedRows` (`pulumi.Input[bool]`) - Accept rows that are missing trailing optional columns. The missing values are treated as nulls.
            If false, records with missing trailing columns are treated as bad records, and if there are too many bad records,
            an invalid error is returned in the job result. The default value is false. Only applicable to CSV, ignored for other formats.
          * `allowQuotedNewlines` (`pulumi.Input[bool]`) - Indicates if BigQuery should allow quoted data sections that contain newline characters in a CSV file.
            The default value is false.
          * `autodetect` (`pulumi.Input[bool]`) - Indicates if we should automatically infer the options and schema for CSV and JSON sources.
          * `createDisposition` (`pulumi.Input[str]`) - Specifies whether the job is allowed to create new tables. The following values are supported:
            CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
            CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
            Creation, truncation and append actions occur as one atomic update upon job completion
          * `destinationEncryptionConfiguration` (`pulumi.Input[dict]`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
            * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
              The BigQuery Service Account associated with your project requires access to this encryption key.

          * `destinationTable` (`pulumi.Input[dict]`) - The destination table.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `encoding` (`pulumi.Input[str]`) - The character encoding of the data. The supported values are UTF-8 or ISO-8859-1.
            The default value is UTF-8. BigQuery decodes the data after the raw, binary data
            has been split using the values of the quote and fieldDelimiter properties.
          * `fieldDelimiter` (`pulumi.Input[str]`) - When extracting data in CSV format, this defines the delimiter to use between fields in the exported data.
            Default is ','
          * `ignoreUnknownValues` (`pulumi.Input[bool]`) - Indicates if BigQuery should allow extra values that are not represented in the table schema.
            If true, the extra values are ignored. If false, records with extra columns are treated as bad records,
            and if there are too many bad records, an invalid error is returned in the job result.
            The default value is false. The sourceFormat property determines what BigQuery treats as an extra value:
            CSV: Trailing columns
            JSON: Named values that don't match any column names
          * `maxBadRecords` (`pulumi.Input[float]`) - The maximum number of bad records that BigQuery can ignore when running the job. If the number of bad records exceeds this value,
            an invalid error is returned in the job result. The default value is 0, which requires that all records are valid.
          * `nullMarker` (`pulumi.Input[str]`) - Specifies a string that represents a null value in a CSV file. The default value is the empty string. If you set this
            property to a custom value, BigQuery throws an error if an
            empty string is present for all data types except for STRING and BYTE. For STRING and BYTE columns, BigQuery interprets the empty string as
            an empty value.
          * `projectionFields` (`pulumi.Input[list]`) - If sourceFormat is set to "DATASTORE_BACKUP", indicates which entity properties to load into BigQuery from a Cloud Datastore backup.
            Property names are case sensitive and must be top-level properties. If no properties are specified, BigQuery loads all properties.
            If any named property isn't found in the Cloud Datastore backup, an invalid error is returned in the job result.
          * `quote` (`pulumi.Input[str]`) - The value that is used to quote data sections in a CSV file. BigQuery converts the string to ISO-8859-1 encoding,
            and then uses the first byte of the encoded string to split the data in its raw, binary state.
            The default value is a double-quote ('"'). If your data does not contain quoted sections, set the property value to an empty string.
            If your data contains quoted newline characters, you must also set the allowQuotedNewlines property to true.
          * `schemaUpdateOptions` (`pulumi.Input[list]`) - Allows the schema of the destination table to be updated as a side effect of the load job if a schema is autodetected or
            supplied in the job configuration. Schema update options are supported in two cases: when writeDisposition is WRITE_APPEND;
            when writeDisposition is WRITE_TRUNCATE and the destination table is a partition of a table, specified by partition decorators.
            For normal tables, WRITE_TRUNCATE will always overwrite the schema. One or more of the following values are specified:
            ALLOW_FIELD_ADDITION: allow adding a nullable field to the schema.
            ALLOW_FIELD_RELAXATION: allow relaxing a required field in the original schema to nullable.
          * `skipLeadingRows` (`pulumi.Input[float]`) - The number of rows at the top of a CSV file that BigQuery will skip when loading the data.
            The default value is 0. This property is useful if you have header rows in the file that should be skipped.
            When autodetect is on, the behavior is the following:
            skipLeadingRows unspecified - Autodetect tries to detect headers in the first row. If they are not detected,
            the row is read as data. Otherwise data is read starting from the second row.
            skipLeadingRows is 0 - Instructs autodetect that there are no headers and data should be read starting from the first row.
            skipLeadingRows = N > 0 - Autodetect skips N-1 rows and tries to detect headers in row N. If headers are not detected,
            row N is just skipped. Otherwise row N is used to extract column names for the detected schema.
          * `sourceFormat` (`pulumi.Input[str]`) - The format of the data files. For CSV files, specify "CSV". For datastore backups, specify "DATASTORE_BACKUP".
            For newline-delimited JSON, specify "NEWLINE_DELIMITED_JSON". For Avro, specify "AVRO". For parquet, specify "PARQUET".
            For orc, specify "ORC". The default value is CSV.
          * `sourceUris` (`pulumi.Input[list]`) - The fully-qualified URIs that point to your data in Google Cloud.
            For Google Cloud Storage URIs: Each URI can contain one '*' wildcard character
            and it must come after the 'bucket' name. Size limits related to load jobs apply
            to external data sources. For Google Cloud Bigtable URIs: Exactly one URI can be
            specified and it has be a fully specified and valid HTTPS URL for a Google Cloud Bigtable table.
            For Google Cloud Datastore backups: Exactly one URI can be specified. Also, the '*' wildcard character is not allowed.
          * `time_partitioning` (`pulumi.Input[dict]`) - Time-based partitioning specification for the destination table.  Structure is documented below.
            * `expirationMs` (`pulumi.Input[str]`) - Number of milliseconds for which to keep the storage for a partition. A wrapper is used here because 0 is an invalid value.
            * `field` (`pulumi.Input[str]`) - If not set, the table is partitioned by pseudo column '_PARTITIONTIME'; if set, the table is partitioned by this field.
              The field must be a top-level TIMESTAMP or DATE field. Its mode must be NULLABLE or REQUIRED.
              A wrapper is used here because an empty string is an invalid value.
            * `type` (`pulumi.Input[str]`) - The only type supported is DAY, which will generate one partition per day. Providing an empty string used to cause an error,
              but in OnePlatform the field will be treated as unset.

          * `writeDisposition` (`pulumi.Input[str]`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
            WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
            WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
            WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
            Creation, truncation and append actions occur as one atomic update upon job completion.

        The **query** object supports the following:

          * `allowLargeResults` (`pulumi.Input[bool]`) - If true and query uses legacy SQL dialect, allows the query to produce arbitrarily large result tables at a slight cost in performance.
            Requires destinationTable to be set. For standard SQL queries, this flag is ignored and large results are always allowed.
            However, you must still set destinationTable when result size exceeds the allowed maximum response size.
          * `createDisposition` (`pulumi.Input[str]`) - Specifies whether the job is allowed to create new tables. The following values are supported:
            CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
            CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
            Creation, truncation and append actions occur as one atomic update upon job completion
          * `defaultDataset` (`pulumi.Input[dict]`) - Specifies the default dataset to use for unqualified table names in the query. Note that this does not alter behavior of unqualified dataset names.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.

          * `destinationEncryptionConfiguration` (`pulumi.Input[dict]`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
            * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
              The BigQuery Service Account associated with your project requires access to this encryption key.

          * `destinationTable` (`pulumi.Input[dict]`) - The destination table.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `flattenResults` (`pulumi.Input[bool]`) - If true and query uses legacy SQL dialect, flattens all nested and repeated fields in the query results.
            allowLargeResults must be true if this is set to false. For standard SQL queries, this flag is ignored and results are never flattened.
          * `maximumBillingTier` (`pulumi.Input[float]`) - Limits the billing tier for this job. Queries that have resource usage beyond this tier will fail (without incurring a charge).
            If unspecified, this will be set to your project default.
          * `maximumBytesBilled` (`pulumi.Input[str]`) - Limits the bytes billed for this job. Queries that will have bytes billed beyond this limit will fail (without incurring a charge).
            If unspecified, this will be set to your project default.
          * `parameterMode` (`pulumi.Input[str]`) - Standard SQL only. Set to POSITIONAL to use positional (?) query parameters or to NAMED to use named (@myparam) query parameters in this query.
          * `priority` (`pulumi.Input[str]`) - Specifies a priority for the query.
          * `query` (`pulumi.Input[str]`) - Configures a query job.  Structure is documented below.
          * `schemaUpdateOptions` (`pulumi.Input[list]`) - Allows the schema of the destination table to be updated as a side effect of the load job if a schema is autodetected or
            supplied in the job configuration. Schema update options are supported in two cases: when writeDisposition is WRITE_APPEND;
            when writeDisposition is WRITE_TRUNCATE and the destination table is a partition of a table, specified by partition decorators.
            For normal tables, WRITE_TRUNCATE will always overwrite the schema. One or more of the following values are specified:
            ALLOW_FIELD_ADDITION: allow adding a nullable field to the schema.
            ALLOW_FIELD_RELAXATION: allow relaxing a required field in the original schema to nullable.
          * `scriptOptions` (`pulumi.Input[dict]`) - Options controlling the execution of scripts.  Structure is documented below.
            * `keyResultStatement` (`pulumi.Input[str]`) - Determines which statement in the script represents the "key result",
              used to populate the schema and query results of the script job.
            * `statementByteBudget` (`pulumi.Input[str]`) - Limit on the number of bytes billed per statement. Exceeding this budget results in an error.
            * `statementTimeoutMs` (`pulumi.Input[str]`) - Timeout period for each statement in a script.

          * `useLegacySql` (`pulumi.Input[bool]`) - Specifies whether to use BigQuery's legacy SQL dialect for this query. The default value is true.
            If set to false, the query will use BigQuery's standard SQL.
          * `useQueryCache` (`pulumi.Input[bool]`) - Whether to look for the result in the query cache. The query cache is a best-effort cache that will be flushed whenever
            tables in the query are modified. Moreover, the query cache is only available when a query does not have a destination table specified.
            The default value is true.
          * `userDefinedFunctionResources` (`pulumi.Input[list]`) - Describes user-defined function resources used in the query.  Structure is documented below.
            * `inlineCode` (`pulumi.Input[str]`) - An inline resource that contains code for a user-defined function (UDF).
              Providing a inline code resource is equivalent to providing a URI for a file containing the same code.
            * `resource_uri` (`pulumi.Input[str]`) - A code resource to load from a Google Cloud Storage URI (gs://bucket/path).

          * `writeDisposition` (`pulumi.Input[str]`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
            WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
            WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
            WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
            Creation, truncation and append actions occur as one atomic update upon job completion.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['copy'] = copy
            __props__['extract'] = extract
            if job_id is None:
                raise TypeError("Missing required property 'job_id'")
            __props__['job_id'] = job_id
            __props__['job_timeout_ms'] = job_timeout_ms
            __props__['labels'] = labels
            __props__['load'] = load
            __props__['location'] = location
            __props__['project'] = project
            __props__['query'] = query
            __props__['job_type'] = None
            __props__['user_email'] = None
        super(Job, __self__).__init__(
            'gcp:bigquery/job:Job',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, copy=None, extract=None, job_id=None, job_timeout_ms=None, job_type=None, labels=None, load=None, location=None, project=None, query=None, user_email=None):
        """
        Get an existing Job resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] copy: Copies a table.  Structure is documented below.
        :param pulumi.Input[dict] extract: Configures an extract job.  Structure is documented below.
        :param pulumi.Input[str] job_id: The ID of the job. The ID must contain only letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). The maximum length is 1,024 characters.
        :param pulumi.Input[str] job_timeout_ms: Job timeout in milliseconds. If this time limit is exceeded, BigQuery may attempt to terminate the job.
        :param pulumi.Input[str] job_type: The type of the job.
        :param pulumi.Input[dict] labels: The labels associated with this job. You can use these to organize and group your jobs.
        :param pulumi.Input[dict] load: Configures a load job.  Structure is documented below.
        :param pulumi.Input[str] location: The geographic location of the job. The default value is US.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] query: Configures a query job.  Structure is documented below.
        :param pulumi.Input[str] user_email: Email address of the user who ran the job.

        The **copy** object supports the following:

          * `createDisposition` (`pulumi.Input[str]`) - Specifies whether the job is allowed to create new tables. The following values are supported:
            CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
            CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
            Creation, truncation and append actions occur as one atomic update upon job completion
          * `destinationEncryptionConfiguration` (`pulumi.Input[dict]`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
            * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
              The BigQuery Service Account associated with your project requires access to this encryption key.

          * `destinationTable` (`pulumi.Input[dict]`) - The destination table.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `sourceTables` (`pulumi.Input[list]`) - Source tables to copy.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `writeDisposition` (`pulumi.Input[str]`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
            WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
            WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
            WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
            Creation, truncation and append actions occur as one atomic update upon job completion.

        The **extract** object supports the following:

          * `compression` (`pulumi.Input[str]`) - The compression type to use for exported files. Possible values include GZIP, DEFLATE, SNAPPY, and NONE.
            The default value is NONE. DEFLATE and SNAPPY are only supported for Avro.
          * `destinationFormat` (`pulumi.Input[str]`) - The exported file format. Possible values include CSV, NEWLINE_DELIMITED_JSON and AVRO for tables and SAVED_MODEL for models.
            The default value for tables is CSV. Tables with nested or repeated fields cannot be exported as CSV.
            The default value for models is SAVED_MODEL.
          * `destinationUris` (`pulumi.Input[list]`) - A list of fully-qualified Google Cloud Storage URIs where the extracted table should be written.
          * `fieldDelimiter` (`pulumi.Input[str]`) - When extracting data in CSV format, this defines the delimiter to use between fields in the exported data.
            Default is ','
          * `printHeader` (`pulumi.Input[bool]`) - Whether to print out a header row in the results. Default is true.
          * `sourceModel` (`pulumi.Input[dict]`) - A reference to the model being exported.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `modelId` (`pulumi.Input[str]`) - The ID of the model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.

          * `sourceTable` (`pulumi.Input[dict]`) - A reference to the table being exported.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `useAvroLogicalTypes` (`pulumi.Input[bool]`) - Whether to use logical types when extracting to AVRO format.

        The **load** object supports the following:

          * `allowJaggedRows` (`pulumi.Input[bool]`) - Accept rows that are missing trailing optional columns. The missing values are treated as nulls.
            If false, records with missing trailing columns are treated as bad records, and if there are too many bad records,
            an invalid error is returned in the job result. The default value is false. Only applicable to CSV, ignored for other formats.
          * `allowQuotedNewlines` (`pulumi.Input[bool]`) - Indicates if BigQuery should allow quoted data sections that contain newline characters in a CSV file.
            The default value is false.
          * `autodetect` (`pulumi.Input[bool]`) - Indicates if we should automatically infer the options and schema for CSV and JSON sources.
          * `createDisposition` (`pulumi.Input[str]`) - Specifies whether the job is allowed to create new tables. The following values are supported:
            CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
            CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
            Creation, truncation and append actions occur as one atomic update upon job completion
          * `destinationEncryptionConfiguration` (`pulumi.Input[dict]`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
            * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
              The BigQuery Service Account associated with your project requires access to this encryption key.

          * `destinationTable` (`pulumi.Input[dict]`) - The destination table.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `encoding` (`pulumi.Input[str]`) - The character encoding of the data. The supported values are UTF-8 or ISO-8859-1.
            The default value is UTF-8. BigQuery decodes the data after the raw, binary data
            has been split using the values of the quote and fieldDelimiter properties.
          * `fieldDelimiter` (`pulumi.Input[str]`) - When extracting data in CSV format, this defines the delimiter to use between fields in the exported data.
            Default is ','
          * `ignoreUnknownValues` (`pulumi.Input[bool]`) - Indicates if BigQuery should allow extra values that are not represented in the table schema.
            If true, the extra values are ignored. If false, records with extra columns are treated as bad records,
            and if there are too many bad records, an invalid error is returned in the job result.
            The default value is false. The sourceFormat property determines what BigQuery treats as an extra value:
            CSV: Trailing columns
            JSON: Named values that don't match any column names
          * `maxBadRecords` (`pulumi.Input[float]`) - The maximum number of bad records that BigQuery can ignore when running the job. If the number of bad records exceeds this value,
            an invalid error is returned in the job result. The default value is 0, which requires that all records are valid.
          * `nullMarker` (`pulumi.Input[str]`) - Specifies a string that represents a null value in a CSV file. The default value is the empty string. If you set this
            property to a custom value, BigQuery throws an error if an
            empty string is present for all data types except for STRING and BYTE. For STRING and BYTE columns, BigQuery interprets the empty string as
            an empty value.
          * `projectionFields` (`pulumi.Input[list]`) - If sourceFormat is set to "DATASTORE_BACKUP", indicates which entity properties to load into BigQuery from a Cloud Datastore backup.
            Property names are case sensitive and must be top-level properties. If no properties are specified, BigQuery loads all properties.
            If any named property isn't found in the Cloud Datastore backup, an invalid error is returned in the job result.
          * `quote` (`pulumi.Input[str]`) - The value that is used to quote data sections in a CSV file. BigQuery converts the string to ISO-8859-1 encoding,
            and then uses the first byte of the encoded string to split the data in its raw, binary state.
            The default value is a double-quote ('"'). If your data does not contain quoted sections, set the property value to an empty string.
            If your data contains quoted newline characters, you must also set the allowQuotedNewlines property to true.
          * `schemaUpdateOptions` (`pulumi.Input[list]`) - Allows the schema of the destination table to be updated as a side effect of the load job if a schema is autodetected or
            supplied in the job configuration. Schema update options are supported in two cases: when writeDisposition is WRITE_APPEND;
            when writeDisposition is WRITE_TRUNCATE and the destination table is a partition of a table, specified by partition decorators.
            For normal tables, WRITE_TRUNCATE will always overwrite the schema. One or more of the following values are specified:
            ALLOW_FIELD_ADDITION: allow adding a nullable field to the schema.
            ALLOW_FIELD_RELAXATION: allow relaxing a required field in the original schema to nullable.
          * `skipLeadingRows` (`pulumi.Input[float]`) - The number of rows at the top of a CSV file that BigQuery will skip when loading the data.
            The default value is 0. This property is useful if you have header rows in the file that should be skipped.
            When autodetect is on, the behavior is the following:
            skipLeadingRows unspecified - Autodetect tries to detect headers in the first row. If they are not detected,
            the row is read as data. Otherwise data is read starting from the second row.
            skipLeadingRows is 0 - Instructs autodetect that there are no headers and data should be read starting from the first row.
            skipLeadingRows = N > 0 - Autodetect skips N-1 rows and tries to detect headers in row N. If headers are not detected,
            row N is just skipped. Otherwise row N is used to extract column names for the detected schema.
          * `sourceFormat` (`pulumi.Input[str]`) - The format of the data files. For CSV files, specify "CSV". For datastore backups, specify "DATASTORE_BACKUP".
            For newline-delimited JSON, specify "NEWLINE_DELIMITED_JSON". For Avro, specify "AVRO". For parquet, specify "PARQUET".
            For orc, specify "ORC". The default value is CSV.
          * `sourceUris` (`pulumi.Input[list]`) - The fully-qualified URIs that point to your data in Google Cloud.
            For Google Cloud Storage URIs: Each URI can contain one '*' wildcard character
            and it must come after the 'bucket' name. Size limits related to load jobs apply
            to external data sources. For Google Cloud Bigtable URIs: Exactly one URI can be
            specified and it has be a fully specified and valid HTTPS URL for a Google Cloud Bigtable table.
            For Google Cloud Datastore backups: Exactly one URI can be specified. Also, the '*' wildcard character is not allowed.
          * `time_partitioning` (`pulumi.Input[dict]`) - Time-based partitioning specification for the destination table.  Structure is documented below.
            * `expirationMs` (`pulumi.Input[str]`) - Number of milliseconds for which to keep the storage for a partition. A wrapper is used here because 0 is an invalid value.
            * `field` (`pulumi.Input[str]`) - If not set, the table is partitioned by pseudo column '_PARTITIONTIME'; if set, the table is partitioned by this field.
              The field must be a top-level TIMESTAMP or DATE field. Its mode must be NULLABLE or REQUIRED.
              A wrapper is used here because an empty string is an invalid value.
            * `type` (`pulumi.Input[str]`) - The only type supported is DAY, which will generate one partition per day. Providing an empty string used to cause an error,
              but in OnePlatform the field will be treated as unset.

          * `writeDisposition` (`pulumi.Input[str]`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
            WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
            WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
            WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
            Creation, truncation and append actions occur as one atomic update upon job completion.

        The **query** object supports the following:

          * `allowLargeResults` (`pulumi.Input[bool]`) - If true and query uses legacy SQL dialect, allows the query to produce arbitrarily large result tables at a slight cost in performance.
            Requires destinationTable to be set. For standard SQL queries, this flag is ignored and large results are always allowed.
            However, you must still set destinationTable when result size exceeds the allowed maximum response size.
          * `createDisposition` (`pulumi.Input[str]`) - Specifies whether the job is allowed to create new tables. The following values are supported:
            CREATE_IF_NEEDED: If the table does not exist, BigQuery creates the table.
            CREATE_NEVER: The table must already exist. If it does not, a 'notFound' error is returned in the job result.
            Creation, truncation and append actions occur as one atomic update upon job completion
          * `defaultDataset` (`pulumi.Input[dict]`) - Specifies the default dataset to use for unqualified table names in the query. Note that this does not alter behavior of unqualified dataset names.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.

          * `destinationEncryptionConfiguration` (`pulumi.Input[dict]`) - Custom encryption configuration (e.g., Cloud KMS keys)  Structure is documented below.
            * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination BigQuery table.
              The BigQuery Service Account associated with your project requires access to this encryption key.

          * `destinationTable` (`pulumi.Input[dict]`) - The destination table.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this model.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this model.
            * `table_id` (`pulumi.Input[str]`) - The table. Can be specified `{{table_id}}` if `project_id` and `dataset_id` are also set,
              or of the form `projects/{{project}}/datasets/{{dataset_id}}/tables/{{table_id}}` if not.

          * `flattenResults` (`pulumi.Input[bool]`) - If true and query uses legacy SQL dialect, flattens all nested and repeated fields in the query results.
            allowLargeResults must be true if this is set to false. For standard SQL queries, this flag is ignored and results are never flattened.
          * `maximumBillingTier` (`pulumi.Input[float]`) - Limits the billing tier for this job. Queries that have resource usage beyond this tier will fail (without incurring a charge).
            If unspecified, this will be set to your project default.
          * `maximumBytesBilled` (`pulumi.Input[str]`) - Limits the bytes billed for this job. Queries that will have bytes billed beyond this limit will fail (without incurring a charge).
            If unspecified, this will be set to your project default.
          * `parameterMode` (`pulumi.Input[str]`) - Standard SQL only. Set to POSITIONAL to use positional (?) query parameters or to NAMED to use named (@myparam) query parameters in this query.
          * `priority` (`pulumi.Input[str]`) - Specifies a priority for the query.
          * `query` (`pulumi.Input[str]`) - Configures a query job.  Structure is documented below.
          * `schemaUpdateOptions` (`pulumi.Input[list]`) - Allows the schema of the destination table to be updated as a side effect of the load job if a schema is autodetected or
            supplied in the job configuration. Schema update options are supported in two cases: when writeDisposition is WRITE_APPEND;
            when writeDisposition is WRITE_TRUNCATE and the destination table is a partition of a table, specified by partition decorators.
            For normal tables, WRITE_TRUNCATE will always overwrite the schema. One or more of the following values are specified:
            ALLOW_FIELD_ADDITION: allow adding a nullable field to the schema.
            ALLOW_FIELD_RELAXATION: allow relaxing a required field in the original schema to nullable.
          * `scriptOptions` (`pulumi.Input[dict]`) - Options controlling the execution of scripts.  Structure is documented below.
            * `keyResultStatement` (`pulumi.Input[str]`) - Determines which statement in the script represents the "key result",
              used to populate the schema and query results of the script job.
            * `statementByteBudget` (`pulumi.Input[str]`) - Limit on the number of bytes billed per statement. Exceeding this budget results in an error.
            * `statementTimeoutMs` (`pulumi.Input[str]`) - Timeout period for each statement in a script.

          * `useLegacySql` (`pulumi.Input[bool]`) - Specifies whether to use BigQuery's legacy SQL dialect for this query. The default value is true.
            If set to false, the query will use BigQuery's standard SQL.
          * `useQueryCache` (`pulumi.Input[bool]`) - Whether to look for the result in the query cache. The query cache is a best-effort cache that will be flushed whenever
            tables in the query are modified. Moreover, the query cache is only available when a query does not have a destination table specified.
            The default value is true.
          * `userDefinedFunctionResources` (`pulumi.Input[list]`) - Describes user-defined function resources used in the query.  Structure is documented below.
            * `inlineCode` (`pulumi.Input[str]`) - An inline resource that contains code for a user-defined function (UDF).
              Providing a inline code resource is equivalent to providing a URI for a file containing the same code.
            * `resource_uri` (`pulumi.Input[str]`) - A code resource to load from a Google Cloud Storage URI (gs://bucket/path).

          * `writeDisposition` (`pulumi.Input[str]`) - Specifies the action that occurs if the destination table already exists. The following values are supported:
            WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the query result.
            WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
            WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            Each action is atomic and only occurs if BigQuery is able to complete the job successfully.
            Creation, truncation and append actions occur as one atomic update upon job completion.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["copy"] = copy
        __props__["extract"] = extract
        __props__["job_id"] = job_id
        __props__["job_timeout_ms"] = job_timeout_ms
        __props__["job_type"] = job_type
        __props__["labels"] = labels
        __props__["load"] = load
        __props__["location"] = location
        __props__["project"] = project
        __props__["query"] = query
        __props__["user_email"] = user_email
        return Job(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

